/*******************************************************************************
 * Copyright (c) 2013-2015 Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * Contributors:
 *     Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza - initial API and implementation
 ******************************************************************************/
package com.net2plan.examples.netDesignAlgorithm.tca;

import com.net2plan.interfaces.networkDesign.IAlgorithm;
import com.net2plan.interfaces.networkDesign.Net2PlanException;
import com.net2plan.interfaces.networkDesign.NetPlan;
import com.net2plan.utils.Triple;
import java.util.*;

/**
 * This algorithms computes the bidirectional minimum spanning tree (MST) for the given set of nodes, using the node distance as the cost measure for each link.
 * For a network of N nodes, the returned topology is a tree of N-1 bidirectional links, so that no other topology is able to connect the N nodes
 * with bidirectional links, at a lower cost (being the cost the sum of the lengths of the links). To compute the MST, the Prim algorithm is implemented.
 *
 * @author Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza
 * @version 1.1, May 2015
 */
public class TCA_PrimMST implements IAlgorithm
{
	@Override
	public String executeAlgorithm(NetPlan netPlan, Map<String, String> algorithmParameters, Map<String, String> net2planParameters)
	{
		/* Basic checks */
		final int N = netPlan.getNumberOfNodes();
		if (N < 2) throw new Net2PlanException("This algorithm requires at least two nodes");

		/* Initialize some variables */
		long initialNodeId = Long.parseLong(algorithmParameters.get("initialNodeId"));
		netPlan.checkNodeIsActive(initialNodeId);

		netPlan.removeAllLinks();
		double linkCapacities = Double.parseDouble(algorithmParameters.get("linkCapacities"));

		Set<Long> nodesNotInTree = new LinkedHashSet<Long>(netPlan.getNodeIds());
		nodesNotInTree.remove(initialNodeId);

		Set<Long> nodesInTree = new LinkedHashSet<Long>();
		nodesInTree.add(initialNodeId);

		while (!nodesNotInTree.isEmpty())
		{
			double minLength = Double.MAX_VALUE;
			long src = -1;
			long dst = -1;

			for (long originNodeId : nodesInTree)
			{
				for (long destinationNodeId : nodesNotInTree)
				{
					double costThisPair = netPlan.getNodePairEuclideanDistance(originNodeId, destinationNodeId);

					if (costThisPair < minLength)
					{
						src = originNodeId;
						dst = destinationNodeId;
						minLength = costThisPair;
					}
				}
			}

			nodesNotInTree.remove(dst);
			nodesInTree.add(dst);

			netPlan.addLinkBidirectional(src, dst, linkCapacities, minLength, null);
		}

		return "Ok!";
	}

	@Override
	public String getDescription()
	{
		return "This algorithms computes the bidirectional minimum spanning tree (MST) for the given set of nodes, using the "
				+ "node distance as the cost measure for each link. For a network of N nodes, the returned topology is a tree of N-1 "
				+ "bidirectional links, so that no other topology is able to connect the N nodes with bidirectional links, at a lower"
				+ " cost (being the cost the sum of the lengths of the links). To compute the MST, the Prim algorithm is implemented.";
	}

	@Override
	public List<Triple<String, String, String>> getParameters()
	{
		List<Triple<String, String, String>> algorithmParameters = new LinkedList<Triple<String, String, String>>();
		algorithmParameters.add(Triple.of("linkCapacities", "100", "The capacities to set in the links"));
		algorithmParameters.add(Triple.of("initialNodeId", "0", "Initial node of the spanning tree"));

		return algorithmParameters;
	}
}